{	This program is copyright (c) 1993 - 2005 Jos Kunst, the Jos Kunst heirs.	}
{	This program is free software. You can redistribute it and/or modify it 	}
{	under the terms of version 2 of the GNU General Public License, which you	}
{	should have received with it.												}
{	This program is distributed in the hope that it will be useful, but			}
{	without any warranty, expressed or implied.									}

unit doposition;
{-------------------------------------------------------------------------------------------------------}
{We use the following schema for the beat position numbers:                                                                                   }
{                                                                                                                                                                            }
{         2                6                10                 14                              every other 32nd                                              }
{0               4                8                  12                   0  --> etc       all & only16ths                                                 }
{     1     3        5      7        9       11        13      15                         triplet 32nds, except the first of each triplet    }
{                                                                                                                                                                            }
{                                                                                                    (all odd numbers are triplet positions!)              }
{                                                                                                                                                                            }
{---------------------------------time is measured in triplet ninetysixths!----------------------------------}

interface
	uses
		translations, chordscanner, chordworker, randomizers, outputs, players, workspace;

	type
		distrchord = array[vl1..vlc] of pitchset;
		chordsarr = array[1..32] of pitchset;

	procedure settozero (var p: position);
{returns the 'empty' position: all places filled with zeros}
	procedure donextposition (lap: position;
									var thch: pitchset;
									dur: byte;
									var thp: position);
{given a starting situation ("time-slice" with duration) and a new chord, }
{gives it its best instrumentation plus the resulting new time-slice}

implementation

	procedure settozero (var p: position);
		var
			r, c: byte;
	begin
		for r := 1 to m do
			for c := 1 to n do
				p[r, c] := 0
	end;

	procedure donextposition (lap: position;
									var thch: pitchset;
									dur: byte;
									var thp: position);

		var
			p, q, r, s, c, cardnoteslots, d1, h1, d2, h2, imnoass, trans, art, dyn: byte;
			adaptstep: integer;
			newch, resultch, leftovertones: pitchset;
			thcr: chordrec;
			problarr: chordarr;
			impossibles: chordsarr;
			i: instruments;
			pass, done: boolean;
			strplayers, remplayers: instrset;
			fixedsubchord, mobilesubchord, oldsubchord, minsecsteps, majsecsteps: distrchord;

		procedure checkremchin (rmch: pitchset;
										var prblarr: chordarr);
{ to avoid changing only the less problematic tones, }
{take stock of how often each of the tones has failed to be matched with an instrument }
{and specify the corresponding ordering }
			var
				p, r, tomove1, tomove2: byte;
				done: boolean;

			function wellordered (car: chordarr): boolean;
				var
					p: byte;
			begin
				p := 1;
				wellordered := TRUE;
				repeat
					p := p + 1;
					if car[p - 1, 2] < car[p, 2] then
						wellordered := FALSE
				until (p = 102) or (car[p, 2] = 0)
			end;

		begin
			for p := bottomnote to topnote do
				if p in rmch then
					begin
						r := 0;
						done := FALSE;
						repeat
							r := r + 1;
							if prblarr[r, 1] = 0 then
								begin
									prblarr[r, 1] := p;
									prblarr[r, 2] := 1;
									done := TRUE
								end
							else if prblarr[r, 1] = p then
								begin
									prblarr[r, 2] := prblarr[r, 2] + 1;
									while not wellordered(prblarr) do
										begin
											if prblarr[r, 2] > prblarr[r - 1, 2] then
												begin
													tomove1 := prblarr[r, 1];
													tomove2 := prblarr[r, 2];
													prblarr[r, 1] := prblarr[r - 1, 1];
													prblarr[r, 2] := prblarr[r - 1, 2];
													prblarr[r - 1, 1] := tomove1;
													prblarr[r - 1, 2] := tomove2
												end;
											r := r - 1;
										end;
									done := TRUE
								end
						until done
					end;
		end;

	begin{donextposition}
		settozero(thp);
		pass := TRUE;
		repeat
			newch := thch;
			strplayers := strqt;
{-----------check for how many notes there is space available-----------}
			cardnoteslots := 0;
			for i := vl1 to vlc do
				cardnoteslots := cardnoteslots + lap[ord(i) * 5 + 1, 8];
{--------updating beat position number---------}
			if pass then
				q := lap[m, 2]
			else
				q := thp[m, 2];
			thp[m, 2] := q + 1;
			if thp[m, 2] = 16 then
				thp[m, 2] := 0;
{----------counting the beats------------}
			if pass then
				q := lap[m - 1, 2]
			else
				q := thp[m - 1, 2];
			if thp[m, 2] = 0 then
				if q = 4 then
					thp[m - 1, 2] := 1
				else
					thp[m - 1, 2] := q + 1
			else
				thp[m - 1, 2] := q;
{---------... and the measure numbers------}
			if pass then
				q := lap[m - 2, 2]
			else
				q := thp[m - 2, 2];
			if (thp[m, 2] = 0) and (thp[m - 1, 2] = 1) then
				thp[m - 2, 2] := q + 1
			else
				thp[m - 2, 2] := q;
{----note slots unavailable:  because of remaining duration commitment.... -----}
{--------------------------- .... or because of binary/ternary commitments-----}
			for i := vl1 to vlc do
				for s := 1 to 4 do
					begin
						if lap[ord(i) * 5 + s, 1] > 0 then
{-------updating & honouring remaining duration value-------}
							if lap[ord(i) * 5 + s, 3] = 255 then
								begin
									thp[ord(i) * 5 + s, 3] := 255;
									thp[ord(i) * 5 + s, 1] := lap[ord(i) * 5 + s, 1];
									cardnoteslots := cardnoteslots - 1;
									if banonunisono then
										newch := newch - [thp[ord(i) * 5 + s, 1]];
{--------never more than two sustained notes per instrument: --------}
									if s > 1 then
										strplayers := strplayers - [i];
{-----------error correcting: all sustained notes belong on s 1 (possibly also 2): -----------}
									if (s > 1) and (thp[ord(i) * 5 + (s - 1), 3] < 255) then
										begin
											p := thp[ord(i) * 5 + (s - 1), 1];
											q := thp[ord(i) * 5 + (s - 1), 3];
											thp[ord(i) * 5 + (s - 1), 1] := thp[ord(i) * 5 + s, 1];
											thp[ord(i) * 5 + (s - 1), 3] := thp[ord(i) * 5 + s, 3];
											thp[ord(i) * 5 + s, 1] := p;
											thp[ord(i) * 5 + s, 3] := q;
											if s > 3 then
												writeln('remdur 255 problem on ', ord(i) * 5 + s : 2)
										end
								end
							else
								begin
									if pass then {first time around}
										q := lap[ord(i) * 5 + s, 3]
									else {second & subsequent times around}
										q := thp[ord(i) * 5 + s, 3];
									if q > 0 then
										begin
{ -----------time is measured in triplet ninetysixths!--------}
											if thp[m, 2] mod 4 < 2 then
												begin
													if q = 1 then
														thp[ord(i) * 5 + s, 3] := 0
													else
														thp[ord(i) * 5 + s, 3] := q - 2
												end
											else
												thp[ord(i) * 5 + s, 3] := q - 1;
{-------if there is now still time to be done then tone and player are stuck with each other----}
											if thp[ord(i) * 5 + s, 3] > 0 then
												begin
													thp[ord(i) * 5 + s, 1] := lap[ord(i) * 5 + s, 1];
													cardnoteslots := cardnoteslots - 1;
{----if the player does his maximum number of tones, he is out --------}
													if s >= lap[ord(i) * 5 + 1, 8] then
														strplayers := strplayers - [i];
													if banonunisono then
														newch := newch - [thp[ord(i) * 5 + s, 1]]
												end
											else
												thp[ord(i) * 5 + s, 1] := 0
										end
								end;
{----- maintaining or deleting binary/ternary commitments --------}
						if pass then
							q := lap[ord(i) * 5 + s, 2]
						else
							q := thp[ord(i) * 5 + s, 2];
						thp[ord(i) * 5 + s, 2] := q;
						if simplify then
							begin
								if thp[m, 2] mod 8 = 0 then
									thp[ord(i) * 5 + s, 2] := 1
							end
						else if thp[m, 2] mod 4 = 0 then
							thp[ord(i) * 5 + s, 2] := 1;
{--------binary/ternary commitments honored------}
						if thp[ord(i) * 5 + s, 2] = 3 then
							if thp[m, 2] mod 4 = 2 then
								strplayers := strplayers - [i]; {---each player in only one tuplet at a time---}
						if thp[ord(i) * 5 + s, 2] = 2 then
							if thp[m, 2] mod 2 = 1 then
								strplayers := strplayers - [i]; {---each player in only one tuplet at a time---}
					end;
{-------- see whether there are not too many notes in the chord-------------}
			if cardnoteslots < card(newch) then
				pass := FALSE
			else
				pass := TRUE;
{-------------in case no two chords are allowed to overlap:-------}
			if nooverlap then
				for i := vl1 to vlc do
					for s := 1 to 4 do
						if (thp[ord(i) * 5 + s, 3] > 0) and (thp[ord(i) * 5 + s, 3] < 255) then
							pass := FALSE;
{---------if simplify is on, the ad hoc booleans are unnecessary--------}
			if simplify then
				begin
					no8ths := FALSE;
					notr8ths := FALSE;
					no16ths := FALSE;
					notr16ths := FALSE;
					no32nds := FALSE;
					notr32nds := FALSE;
				end;
{----- even geen triooltweendertigsten... -------}
			if notr32nds then
				if (thp[m, 2] mod 8 = 1) or (thp[m, 2] mod 8 = 7) then
					pass := FALSE;
{---- ... en geen tweenderstigsten...... ----------}
			if no32nds then
				if thp[m, 2] mod 4 = 2 then
					pass := FALSE;
{------- ...en geen trioolzestienden ------}
			if notr16ths then
				if thp[m, 2] mod 10 = 3 then
					pass := FALSE;
{------ ....en uberhaupt geen zestienden meer!...-------}
			if no16ths then
				if thp[m, 2] mod 8 = 4 then
					pass := FALSE;
{----------....en ook geen trioolachtsten...-------}
			if notr8ths then
				if thp[m, 2] mod 6 = 5 then
					pass := FALSE;
{------- ....en geen achtsten.......---------}
			if no8ths then
				if thp[m, 2] = 8 then
					pass := FALSE;
{    if not pass then    writeln('no pass; ', cardnoteslots : 1, ' places do not accomodate ', card(newch) : 1, ' notes!')}
		until pass;
{=====================================================================================}
{----------...at last!!! - assigning pitches -------------------}
		thp[1, 8] := maxcardvl1;
		thp[6, 8] := maxcardvl2;
		thp[11, 8] := maxcardvla;
		thp[16, 8] := maxcardvlc;
		for i := vl1 to vlc do
			begin {---- take stock of inherited notes we will be adding to ------}
				thp[ord(i) * 5 + 2, 8] := 0;
				fixedsubchord[i] := [];
				for s := 1 to 4 do
					if thp[ord(i) * 5 + s, 1] > 0 then
						begin
							fixedsubchord[i] := fixedsubchord[i] + [thp[ord(i) * 5 + s, 1]];
							thp[ord(i) * 5 + 2, 8] := thp[ord(i) * 5 + 2, 8] + 1
						end
			end;
		if lotohi then
			begin
				maxcont := FALSE;
{    writeln('lotohi!!')  --> for testdifficulties}
			end;
		if maxcont then
			begin
				lotohi := FALSE;
{    writeln('maxcont!!') --> for testdifficulties}
			end;
{=====================================================================================}
		if lotohi then  {-----...first, (a) pitches distributed equally over the instruments: low-high  vlc-vl1 ------}
			begin
{----distribute the still-to-be-assigned notes over the instruments so as }
{ to produce equality in as far as maxcard(i) allows; store results in card(i) slots-----}
				c := card(newch);
				i := vl1;
				s := 1;
				repeat
					if thp[ord(i) * 5 + s, 1] = 0 then
						if c > 0 then {--- still notes to be assigned? ---}
							if i in strplayers then
								if (thp[ord(i) * 5 + 1, 8] - thp[ord(i) * 5 + 2, 8]) > 0 then {--- still room available? ---}
									begin
										thp[ord(i) * 5 + 2, 8] := thp[ord(i) * 5 + 2, 8] + 1;
										c := c - 1
									end;
					if i < vlc then
						i := succ(i)
					else
						begin
							i := vl1;
							s := s + 1
						end
				until (c = 0) or (s = 5);
{----next: specify the instruments' subchords, taking into account the inherited notes---}
				for i := vl1 to vlc do
					if i in strplayers then
						begin
							p := topnote;
							mobilesubchord[i] := [];
							while (card(fixedsubchord[i] + mobilesubchord[i]) < thp[ord(i) * 5 + 2, 8]) and (p >= ensemblebottom) do
								begin
									if p in newch then
										begin
											newch := newch - [p];
											mobilesubchord[i] := mobilesubchord[i] + [p]
										end;
									p := p - 1
								end;
{    write('fixed subchord has ');    for p := bottomnote to topnote do    if p in fixedsubchord[i] then}
{    write(nameof(p), ' ');    writeln(' for ', name(i));}
{    write('mobile subchord has '); for p := bottomnote to topnote do if p in mobilesubchord[i] then}
{    write(nameof(p), ' ');    writeln(' for ', name(i));}
							resolvedifficulties(fixedsubchord[i], mobilesubchord[i], leftovertones, i, thp[ord(i) * 5 + 1, 8]);
							for s := 1 to 4 do {... and store them in the position matrix ----}
								if thp[ord(i) * 5 + s, 1] = 0 then
									begin
										p := topnote;
										repeat
											p := p - 1
										until (p in mobilesubchord[i]) or (p < ensemblebottom);
										if p in mobilesubchord[i] then
											begin
												thp[ord(i) * 5 + s, 1] := p;
												mobilesubchord[i] := mobilesubchord[i] - [p];
												fixedsubchord[i] := fixedsubchord[i] + [p]
											end
									end;
							newch := newch + leftovertones { --- don't forget the leftovers, except if none can take them in ---}
						end
			end
{=====================================================================================}
		else if maxcont then {---- next: pitches assigned to instruments in other ways, (b) and (c), and/or (d): ----}
			begin
				for i := vl1 to vlc do {--------  (b) instruments may just repeat, or prolong, notes; or ... ----------}
					if i in strplayers then
						begin
{--------- read in the recent but foregone chord -----------}
							oldsubchord[i] := [];
							for s := 1 to 4 do
								if lap[ord(i) * 5 + s, 1] > 0 then
									oldsubchord[i] := oldsubchord[i] + [lap[ord(i) * 5 + s, 1]];
{    write('oldsubchord for ', name(i), ': ');    for p := bottomnote to topnote do}
{    if p in oldsubchord[i] then    write(nameof(p), ' ');    writeln;}
{    write('fixedsubchord for ', name(i), ': ');    for p := bottomnote to topnote do}
{    if p in fixedsubchord[i] then    write(nameof(p), ' ');    writeln;}
{ -------- forget about the notes already in place -------------}
							oldsubchord[i] := oldsubchord[i] - fixedsubchord[i];
							for p := ensembletop downto ensemblebottom do
								if p in oldsubchord[i] * newch then
									if card(fixedsubchord[i]) < thp[ord(i) * 5 + 1, 8] then
										begin
											checkdifficulty(fixedsubchord[i] + [p], i, d1, h1, d2, h2);
{    writeln('difficulty: ', d1 : 1);}
											if d1 < 255 then
												begin {----- store tone in position matrix ---- }
													newch := newch - [p];
													fixedsubchord[i] := fixedsubchord[i] + [p];
													s := 0;
													repeat
														s := s + 1
													until thp[ord(i) * 5 + s, 1] = 0;
													thp[ord(i) * 5 + s, 1] := p
												end
										end
						end;
				for i := vl1 to vlc do {--------  (c) instruments may take small steps; and lastly ... ----------}
					if i in strplayers then
						for p := ensemblebottom to ensembletop do
							if p in newch then
								for q := ensemblebottom to ensembletop do
									if (q in oldsubchord[i]) and not (q in newch) then
										begin
											if abs(p - q) = 1 then
												if card(fixedsubchord[i]) < thp[ord(i) * 5 + 1, 8] then
													begin
														checkdifficulty(fixedsubchord[i] + [p], i, d1, h1, d2, h2);
														if d1 < 255 then
															begin {----- store tone in position matrix ---- }
																newch := newch - [p];
																fixedsubchord[i] := fixedsubchord[i] + [p];
																s := 0;
																repeat
																	s := s + 1
																until thp[ord(i) * 5 + s, 1] = 0;
																thp[ord(i) * 5 + s, 1] := p
															end
													end;
											if abs(p - q) = 2 then
												if card(fixedsubchord[i]) < thp[ord(i) * 5 + 1, 8] then
													begin
														checkdifficulty(fixedsubchord[i] + [p], i, d1, h1, d2, h2);
														if d1 < 255 then
															begin {----- store tone in position matrix ---- }
																newch := newch - [p];
																fixedsubchord[i] := fixedsubchord[i] + [p];
																s := 0;
																repeat
																	s := s + 1
																until thp[ord(i) * 5 + s, 1] = 0;
																thp[ord(i) * 5 + s, 1] := p
															end
													end
										end
			end;
{=====================================================================================}
{----------------    taking stock of what special strategies did for us   ----------------}
		for i := vl1 to vlc do
			if thp[ord(i) * 5 + 1, 8] <= card(fixedsubchord[i]) then
				strplayers := strplayers - [i];
		resultch := [];
		for i := vl1 to vlc do
			resultch := resultch + fixedsubchord[i];
		newch := newch - resultch;
{=====================================================================================}
		if newch <> [] then
			begin
{------ initialize the array of really impossible chords ---------}
				for p := 1 to 32 do
					impossibles[p] := [];
				repeat {---------... (d) notes may be freely (re)distributed over the players -------}
					p := 0;
					i := vl1;
					repeat
						if i in strplayers then
							p := p + 1;
						if i = vlc then
							i := vl1
						else
							i := succ(i)
					until i = vl1;
					r := randnum(p);
					i := vlc;
					if r > 0 then
						repeat
							if i = vlc then
								i := vl1
							else
								i := succ(i);
							if i in strplayers then
								r := r - 1;
						until (r = 0) or (i = vlc);
					if i in strplayers then
						begin
							givitapart(i, newch, q);
{ ------ avoid working twice on the same chord -------}
							p := 0;
							repeat
								p := p + 1
							until (impossibles[p] = fixedsubchord[i] + [q]) or (p = 32);
							if p = 32 then
								begin
									mobilesubchord[i] := [q];
									resolvedifficulties(fixedsubchord[i], mobilesubchord[i], leftovertones, i, thp[ord(i) * 5 + 1, 8]);
									if leftovertones = [] then
										begin {----- store tone in position matrix ---- }
											if card(fixedsubchord[i]) + 1 = thp[ord(i) * 5 + 1, 8] then
												strplayers := strplayers - [i];
											newch := newch - [q];
											fixedsubchord[i] := fixedsubchord[i] + [q];
											s := 0;
											repeat
												s := s + 1
											until thp[ord(i) * 5 + s, 1] = 0;
											thp[ord(i) * 5 + s, 1] := q
										end
									else
										begin
											p := 0;
											repeat
												p := p + 1
											until impossibles[p] = [];
											impossibles[p] := fixedsubchord[i] + [q]
										end
								end
							else
								begin
{------- we are now on the 1st occurrence of this particular impossible chord in the array; ----------}
{----------------- add the new occurrence ----------------------}
									repeat
										p := p + 1
									until impossibles[p] = [];
									impossibles[p] := fixedsubchord[i] + [q];
{--------------- check how many times this chord came up already ---------------}
									imnoass := 0;
									for p := 1 to 32 do
										if impossibles[p] = fixedsubchord[i] + [q] then
											imnoass := imnoass + 1;
								end
						end
				until ((newch = []) or (strplayers = [])) or (imnoass > 8)
			end;

		thp[2, 8] := card(fixedsubchord[vl1]);
		thp[7, 8] := card(fixedsubchord[vl2]);
		thp[12, 8] := card(fixedsubchord[vla]);
		thp[17, 8] := card(fixedsubchord[vlc]);
{------- take in what has become of the chord ---------------}
		for i := vl1 to vlc do
			begin
				checkdifficulty(fixedsubchord[i], i, d1, h1, d2, h2);
				thp[ord(i) * 5 + 5, 1] := h1;
				thp[ord(i) * 5 + 5, 3] := h2;
				thp[ord(i) * 5 + 5, 2] := d1;
				thp[ord(i) * 5 + 5, 4] := d2;
				resultch := resultch + fixedsubchord[i]
			end;
{----transmit chord changes to the program's further workstages!-----}
		thch := resultch;
		scan(thch, thcr);
{------ zzzzzz  }

{-------  encode & store new binary/ternary commitments  ------}
		for i := vl1 to vlc do
			for s := 1 to 4 do
				if thp[ord(i) * 5 + s, 1] <> lap[ord(i) * 5 + s, 1] then
					begin
						if thp[m, 2] mod 4 = 2 then
							thp[ord(i) * 5 + s, 2] := 2;
						if thp[m, 2] mod 2 = 1 then
							thp[ord(i) * 5 + s, 2] := 3;
					end;
{----------store new duration data--------}
		for i := vl1 to vlc do
			for s := 1 to 4 do
				if (thp[ord(i) * 5 + s, 1] > 0) and (thp[ord(i) * 5 + s, 3] = 0) then
					if dur > 0 then
						if simplify then
							thp[ord(i) * 5 + s, 3] := simplified(dur, thp[m, 2])
						else
							thp[ord(i) * 5 + s, 3] := dur;
		thp[m - 1, 4] := round(basedur);
{-------- store dsg data into position array -------}
		thp[m, 4] := dsg;
		thp[m - 2, 4] := adsg;
		thp[m, 1] := hroot;
		p := 1;
		repeat
			for q := 1 to 14 do
				if thp[q, 1] = carr[p, 1] then
					thp[q, 4] := carr[p, 2];
			p := p + 1;
		until carr[p, 1] = 0;
{-------------------------------transartdyn...-------------------------------}
		p := 1;
		repeat
{------do trans: the transfromprecedent/transdirection, and store them--------}
			dodoubledicing(transtarget, transprecision, trans);
			if thp[p, 1] > 0 then
				thp[p, 5] := trans;
{-------do art: pedal&leg/density/tremoli etc-----------------}
			dodoubledicing(arttarget, artprecision, art);
			if thp[p, 1] > 0 then
				thp[p, 6] := art;
{----------do the dynamics: chorddyn/dyncontour ---------------------------------}
			dodoubledicing(dyntarget, dynprecision, dyn);
			if thp[p, 1] > 0 then
				thp[p, 7] := dyn;
			p := p + 5
		until p > 20;
		thp[m - 1, 5] := transtarget;
		thp[m - 2, 5] := transprecision;
		thp[m - 1, 6] := arttarget;
		thp[m - 2, 6] := artprecision;
		thp[m - 1, 7] := dyntarget;
		thp[m - 2, 7] := dynprecision;
{---------------------------------------(constraints on values)-----------------------}
		if (melfix < bottomnote) or (melfix > topnote) then
			if melfix > 1 then
				melfix := 0;
{---------- store jump room value, melodic anchor, and music atom values----------}
		thp[m - 2, 1] := play;
		thp[m - 1, 1] := melfix;
		thp[m - 2, 3] := pitchatom;
		thp[m - 1, 3] := duratom;
		thp[m, 3] := dynatom;
{---------------------------------------(constraints on booleans)-----------------------}
		if lotohi then
			maxcont := FALSE;
		if maxcont then
			lotohi := FALSE;
		if simplify then
			notr32nds := FALSE;
		if notr32nds = FALSE then
			no32nds := FALSE;
		if no32nds = FALSE then
			notr16ths := FALSE;
		if notr16ths = FALSE then
			no16ths := FALSE;
		if no16ths = FALSE then
			notr8ths := FALSE;
		if notr8ths = FALSE then
			no8ths := FALSE;
		if no8ths then
			notr8ths := TRUE;
		if notr8ths then
			no16ths := TRUE;
		if no16ths then
			notr16ths := TRUE;
		if notr16ths then
			no32nds := TRUE;
		if no32nds then
			notr32nds := TRUE;
{------------store all booleans ----------------------------}
		if notr32nds then
			thp[m - 3, 2] := 1
		else
			thp[m - 3, 2] := 2;
		if no32nds then
			thp[m - 3, 3] := 1
		else
			thp[m - 3, 3] := 2;
		if notr16ths then
			thp[m - 3, 4] := 1
		else
			thp[m - 3, 4] := 2;
		if no16ths then
			thp[m - 3, 5] := 1
		else
			thp[m - 3, 5] := 2;
		if notr8ths then
			thp[m - 3, 6] := 1
		else
			thp[m - 3, 6] := 2;
		if no8ths then
			thp[m - 3, 7] := 1
		else
			thp[m - 3, 7] := 2;
		if nooverlap then
			thp[m - 5, 8] := 1
		else
			thp[m - 5, 8] := 2;
		if simplify then
			thp[m - 4, 8] := 1
		else
			thp[m - 4, 8] := 2;
		if preferthirds then
			thp[m - 3, 8] := 1
		else
			thp[m - 3, 8] := 2;
		if maxcont then
			thp[m - 2, 8] := 1
		else
			thp[m - 2, 8] := 2;
		if lotohi then
			thp[m - 1, 8] := 1
		else
			thp[m - 1, 8] := 2;
		if banonunisono then
			thp[m, 8] := 1
		else
			thp[m, 8] := 2
	end;{donextposition}

end.