{	This program is copyright (c) 1993 - 2005 Jos Kunst, the Jos Kunst heirs.	}
{	This program is free software. You can redistribute it and/or modify it 	}
{	under the terms of version 2 of the GNU General Public License, which you	}
{	should have received with it.												}
{	This program is distributed in the hope that it will be useful, but			}
{	without any warranty, expressed or implied.									}

unit players;

interface
	uses
		translations, chordscanner, chordworker, randomizers, outputs;

	function card (cs: pitchset): byte;
{returns the number of notes in a given chord}
	function centerofmassof (ch: pitchset;
									fromnote, tonote: byte): byte;
{returns the best center of (part of) a given chord}
	function handsloxof (cs: pitchset): pitchset;
{returns the middle points of the two hands' best locations for a given chord to be played ("accord plaqu")}
	function RHacceptable (RHs: pitchset): boolean;
{checks whether chord can be done by right hand only}
	function LHacceptable (LHs: pitchset): boolean;
{checks whether chord can be done by left hand only}
	procedure gripinhands (var handsset: pitchset;
									var LHset, RHset: pitchset);
{transforms any chord into one fitting exactly under one pair of pianist's hands}
	procedure givelimitsof (i: instruments;
									var lst, l, h, hst: byte);
{returns the possible and best registers of the given instrument}
	procedure checkdifficulty (targetch: pitchset;
									strins: instruments;
									var difficulty1, handposition1, difficulty2, handposition2: byte);
{calculates the extreme halfpositions and the warnings concerning them }
{of at most two fingerings of the given chord}

	var
		LHwidth, RHwidth, vlap: byte;
		LHout, RHout: boolean;

implementation

	function card (cs: pitchset): byte;
		var
			p, ca: byte;
	begin
		ca := 0;
		for p := bottomnote to topnote do
			if p in cs then
				ca := ca + 1;
		card := ca
	end;

	function centerofmassof (ch: pitchset;
									fromnote, tonote: byte): byte;
		var
			p, lastp, secondpastp, centerofmass, cofm: byte;
			totalweight, twght: integer;
	begin
		totalweight := 0;
		centerofmass := 0;
		lastp := 0;
		for p := fromnote to tonote do
			if p in ch then
				begin
{write(p : 2, ' ');}
					if centerofmass = 0 then { i.e.: this is the first (lowest) chordnote }
						centerofmass := p
					else if totalweight = 0 then { i.e.: this is the second (lowest but one) chordnote }
						begin
							totalweight := p - lastp;
							centerofmass := centerofmass + round((p - lastp) / 2);
						end
					else { i.e.: from the third chordnote onwards }
						begin
							cofm := centerofmass;
							twght := totalweight;
							totalweight := totalweight + (p - lastp);
							centerofmass := centerofmass + round((((p - lastp) / 2) / totalweight) * (lastp - centerofmass));
							centerofmass := centerofmass + round((((p - lastp) / 2) / totalweight) * (p - centerofmass));
							if centerofmass = cofm then { solving a problem occurring with larger clusters... }
								totalweight := twght;
						end;
					lastp := p
				end;
{writeln('--> ', centerofmass : 2);}
		centerofmassof := centerofmass;
	end;

	function handsloxof (cs: pitchset): pitchset;
		var
			center: byte;
	begin
		center := centerofmassof(cs, bottomnote, topnote);
{writeln('center is ', center : 2);}
		if LHout or RHout then
			handsloxof := [center]
		else
			handsloxof := [centerofmassof(cs, bottomnote, center), centerofmassof(cs, center + 1, topnote)]
	end;

	function RHacceptable (RHs: pitchset): boolean;
		var
			p, a, b, c, d, e: byte;
	begin
		a := 0;
		b := 0;
		c := 0;
		d := 0;
		e := 0;
		for p := bottomnote to topnote do
			if p in RHs then
				begin
					if e = d then
						e := p;
					if d = c then
						d := p;
					if c = b then
						c := p;
					if b = a then
						b := p;
					if a = 0 then
						a := p
				end;
		RHacceptable := TRUE;
		if e - d > 3 then
			RHacceptable := FALSE;
		if e - c > 6 then
			RHacceptable := FALSE;
		if e - b > 9 then
			RHacceptable := FALSE;
		if e - a > 12 then
			RHacceptable := FALSE
	end;

	function LHacceptable (LHs: pitchset): boolean;
		var
			p, a, b, c, d, e: byte;
	begin
		a := 0;
		b := 0;
		c := 0;
		d := 0;
		e := 0;
		for p := topnote downto bottomnote do
			if p in LHs then
				begin
					if e = d then
						e := p;
					if d = c then
						d := p;
					if c = b then
						c := p;
					if b = a then
						b := p;
					if a = 0 then
						a := p
				end;
		LHacceptable := TRUE;
		if d - e > 3 then
			LHacceptable := FALSE;
		if c - e > 6 then
			LHacceptable := FALSE;
		if b - e > 9 then
			LHacceptable := FALSE;
		if a - e > 12 then
			LHacceptable := FALSE
	end;

	procedure gripinhands (var handsset: pitchset;
									var LHset, RHset: pitchset);
		var
			LHcenter, RHcenter, p, q: byte;
			handslox, oldset: pitchset;
			newcr: chordrec;

		procedure adapttoRH (var h: pitchset);
			var
				origh, changedh, goodh: pitchset;
				dens, p, q, r, octaveds, ch, change: byte;
				oct: boolean;
		begin
			origh := h;
			if card(origh) > 5 then
				h := []
			else
				begin
					goodh := [];
					changedh := origh;
					octaveds := 0;
					change := 24;
					ch := 0;
					p := bottomnote - 1;
					oct := FALSE;
					if dens > 1 then
						repeat
							p := p + 1;
							if p in changedh then
								begin
									changedh := changedh - [p];
									q := p + 12;
									if q in changedh then
										oct := TRUE;
									changedh := changedh + [q];
									octaveds := octaveds + 1;
									r := p;
									repeat
										r := r + 1
									until r in changedh;
									ch := ch + (r - p);
									if not oct then
										begin
											r := q;
											repeat
												r := r - 1
											until r in changedh;
											ch := ch + (q - r);
										end
									else
										oct := FALSE;
									if RHacceptable(changedh) then
										if ch < change then
											begin
												goodh := changedh;
												change := ch
											end
								end;
						until octaveds >= dens - (dens div 2);
					changedh := origh;
					octaveds := 0;
					ch := 0;
					p := topnote + 1;
					oct := FALSE;
					if dens > 1 then
						repeat
							p := p - 1;
							if p in changedh then
								begin
									changedh := changedh - [p];
									q := p - 12;
									if q in changedh then
										oct := TRUE;
									changedh := changedh + [q];
									octaveds := octaveds + 1;
									r := p;
									repeat
										r := r - 1
									until r in changedh;
									ch := ch + (p - r);
									if not oct then
										begin
											r := q;
											repeat
												r := r + 1
											until r in changedh;
											ch := ch + (r - q)
										end
									else
										oct := FALSE;
									if RHacceptable(changedh) then
										if ch < change then
											begin
												goodh := changedh;
												change := ch
											end
								end;
						until octaveds >= dens - (dens div 2);
					h := goodh
				end
		end;

		procedure adapttoLH (var h: pitchset);
			var
				origh, changedh, goodh: pitchset;
				dens, p, q, r, octaveds, ch, change: byte;
				oct: boolean;
		begin
			origh := h;
			if card(origh) > 5 then
				h := []
			else
				begin
					goodh := [];
					changedh := origh;
					octaveds := 0;
					change := 24;
					ch := 0;
					p := bottomnote - 1;
					oct := FALSE;
					if dens > 1 then
						repeat
							p := p + 1;
							if p in changedh then
								begin
									changedh := changedh - [p];
									q := p + 12;
									if q in changedh then
										oct := TRUE;
									changedh := changedh + [q];
									octaveds := octaveds + 1;
									r := p;
									repeat
										r := r + 1
									until r in changedh;
									ch := ch + (r - p);
									if not oct then
										begin
											r := q;
											repeat
												r := r - 1
											until r in changedh;
											ch := ch + (q - r);
										end
									else
										oct := FALSE;
									if LHacceptable(changedh) then
										if ch < change then
											begin
												goodh := changedh;
												change := ch
											end
								end;
						until octaveds >= dens - (dens div 2);
					changedh := origh;
					octaveds := 0;
					ch := 0;
					p := topnote + 1;
					oct := FALSE;
					if dens > 1 then
						repeat
							p := p - 1;
							if p in changedh then
								begin
									changedh := changedh - [p];
									q := p - 12;
									if q in changedh then
										oct := TRUE;
									changedh := changedh + [q];
									octaveds := octaveds + 1;
									r := p;
									repeat
										r := r - 1
									until r in changedh;
									ch := ch + (p - r);
									if not oct then
										begin
											r := q;
											repeat
												r := r + 1
											until r in changedh;
											ch := ch + (r - q)
										end
									else
										oct := FALSE;
									if LHacceptable(changedh) then
										if ch < change then
											begin
												goodh := changedh;
												change := ch
											end
								end;
						until octaveds >= dens - (dens div 2);
					h := goodh
				end
		end;

	begin
		oldset := handsset;
		handslox := handsloxof(handsset);
		LHset := [];
		RHset := [];
{------- in the case of LHout or RHout we get only one pitch in handslox! --------}
		if RHout then
			begin
				LHcenter := bottomnote - 1;
				repeat
					LHcenter := LHcenter + 1
				until LHcenter in handslox;
				for p := bottomnote to topnote do
					if p in handsset then
						if p < LHcenter then
							begin
								q := p;
								while q < LHcenter - (LHwidth div 2) do
									q := q + 12;
								LHset := LHset + [q]
							end
						else
							begin
								q := p;
								while q > LHcenter + (LHwidth div 2) do
									q := q - 12;
								LHset := LHset + [q]
							end;
			end
		else if LHout then
			begin
				RHcenter := bottomnote - 1;
				repeat
					RHcenter := RHcenter + 1
				until RHcenter in handslox;
				for p := bottomnote to topnote do
					if p in handsset then
						if p < RHcenter then
							begin
								q := p;
								while q < RHcenter - (RHwidth div 2) do
									q := q + 12;
								RHset := RHset + [q]
							end
						else
							begin
								q := p;
								while q > RHcenter + (RHwidth div 2) do
									q := q - 12;
								RHset := RHset + [q]
							end
			end
		else
			begin
				LHcenter := bottomnote - 1;
				repeat
					LHcenter := LHcenter + 1
				until LHcenter in handslox;
				RHcenter := LHcenter;
				repeat
					RHcenter := RHcenter + 1
				until RHcenter in handslox;
				LHset := [];
				RHset := [];
				for p := bottomnote to topnote do
					if p in handsset then
						if p < LHcenter then
							begin
								q := p;
								while q < LHcenter - (LHwidth div 2) do
									q := q + 12;
								LHset := LHset + [q]
							end
						else if abs(p - LHcenter) < abs(RHcenter - p) then
							begin
								q := p;
								while q > LHcenter + (LHwidth div 2) do
									q := q - 12;
								LHset := LHset + [q]
							end
						else if p < RHcenter then
							begin
								q := p;
								while q < RHcenter - (RHwidth div 2) do
									q := q + 12;
								RHset := RHset + [q]
							end
						else
							begin
								q := p;
								while q > RHcenter + (RHwidth div 2) do
									q := q - 12;
								RHset := RHset + [q]
							end
			end;
		if not RHacceptable(RHset) then
			adapttoRH(RHset);
		if not LHacceptable(LHset) then
			adapttoLH(LHset);
		handsset := LHset + RHset;
		if handsset <> oldset then
			begin
				scan(handsset, newcr);
				dsg := newcr.diss;
				hroot := newcr.hifu;
				carr := newcr.stru;
				totalofall := newcr.total
			end
	end;

	procedure givelimitsof (i: instruments;
									var lst, l, h, hst: byte);
	begin
		case i of
{    fl: begin lst := middlec;l := middlec + 2;h := middlec + 33; hst := middlec + 36 end;}
{    ob: begin    lst := middlec - 2;    l := middlec + 4;    h := middlec + 26;    hst := middlec + 33    end;    }
{    cl:     begin    lst := middlec - 10;    l := middlec - 10;    h := middlec + 27;    hst := middlec + 34    end;    }
{    fag :    begin    lst := middlec - 26;    l := middlec - 22;    h := middlec + 10;    hst := middlec + 16    end;}
{    cor:     begin    lst := middlec - 31;    l := middlec - 26;    h := middlec + 17;    hst := middlec + 22    end;}
{   trp: begin    lst := middlec - 9;    l := middlec - 8;    h := middlec + 24;    hst := middlec + 26    end;}
{trbn:     begin    lst := middlec - 29;    l := middlec - 20;    h := middlec + 14;    hst := middlec + 17    end;}
{    sax1:     begin    lst := middlec - 4;    l := middlec - 2;    h := middlec + 25;    hst := middlec + 27    end;    }
{    sax2:     begin    lst := middlec - 11;    l := middlec - 9;    h := middlec + 20;    hst := middlec + 24    end;}
{    sax3:     begin    lst := middlec - 16;    l := middlec - 14;    h := middlec + 15;    hst := middlec + 19    end;    }
{    sax4:     begin    lst := middlec - 23;    l := middlec - 21;    h := middlec + 07;    hst := middlec + 07    end;    }
{    vla1:     begin    lst := middlec - 12;    l := middlec - 12;    h := middlec + 24;    hst := middlec + 33    end;    }
{vla2:     if vlap > 0 then}
{calculate the 2nd vlapitch's range from the first... }
{    begin    lst := vlap - 14;    if vlap < middlec - 5 then    lst := vlap + 1;    if lst < middlec - 12 then    lst := middlec - 12;    l := lst;   }
{ hst := vlap + 14;    if hst > middlec + 24 then    hst := middlec + 24;    h := hst    end    else}
{... if there is no first, do not give any pitch to vla2 }
{    begin    lst := middlec + 51;    l := lst;    hst := middlec - 51;    h := hst    end;}
{ pf: begin lst := middlec - 39; l := middlec - 39; h := middlec + 48; hst := middlec + 48 end;}
			vl1: 
				begin {--- 1st violin: the full four octaves from g3 to g7-----}
					lst := middlec - 5;
					l := middlec - 5;
					h := middlec + 40;
					hst := middlec + 43
				end;
			vl2: 
				begin     {--- 2nd vln: from g3 to e7 ---}
					lst := middlec - 5;
					l := middlec - 5;
					h := middlec + 36;
					hst := middlec + 40
				end;
			vla: 
				begin {--- vla: from c3 to e6 ---}
					lst := middlec - 12;
					l := middlec - 12;
					h := middlec + 24;
					hst := middlec + 28
				end;
			vlc: 
				begin {---- vlc: from c2 to c6 ----}
					lst := middlec - 24;
					l := middlec - 24;
					h := middlec + 21;
					hst := middlec + 24
				end;
		end
	end;

	procedure checkdifficulty (targetch: pitchset;
									strins: instruments;
									var difficulty1, handposition1, difficulty2, handposition2: byte);

		type
			lefthand = array[1..4] of byte;
			alefthand = array[1..5] of integer;
			resultlist = array[1..8] of alefthand;
		var
			halfposition, f1, f2, f3, f4, i, ii, j, ij, str, incr2, incr3, incr4, s, a, b, counter: byte;
			p, lowest, low, high, highest: byte;
			alerts: integer;
			pset, dozeroson, exceptstrings, fiftheds, tuningset: pitchset;
			tuning: array[1..4] of byte;
			fingering: lefthand; { de vier snaren; ze krijgen positiegetallen toegewezen }
			inventory: resultlist;
			dofifths, taskdone, nolefthand: boolean;

		procedure giveprelposfor (cs: pitchset;
										var ps: pitchset);
			var
				tar: set of byte;
				p, pp, q, pos, posit, str, strings, sstrings: byte;
				chord, cord: array[1..4] of byte;
				prelfin: lefthand;
				testpositive: boolean;
				compexceptstrings: pitchset;

			procedure closegap (var pr: lefthand);
				var
					str, sstr: byte;
					somenote, somegap: boolean;
			begin
				str := 5;
				somenote := FALSE;
				repeat
					str := str - 1;
					if pr[str] < 99 then
						somenote := TRUE
				until somenote or (str = 1);
				if somenote then
					begin
						repeat
							str := str - 1;
							if pr[str] = 99 then
								begin
									somegap := FALSE;
									sstr := str;
									repeat
										sstr := sstr - 1;
										if sstr > 0 then
											if pr[sstr] < 99 then
												somegap := TRUE
									until (sstr <= 1) or somegap;
									if somegap then
										begin
											pr[str] := pr[sstr] + (str - sstr) * 7;
											pr[sstr] := 99
										end
								end
						until str <= 2
					end
			end;

		begin{giveprelposfor}
{---------------put chord into array 'chord'--------------}
			for p := 1 to 4 do
				chord[p] := 0;
			tar := cs;
			ps := [];
			for p := topnote downto bottomnote do
				if p in tar then
					begin
						if chord[1] = 0 then
							begin
								chord[1] := p;
								tar := tar - [p]
							end;
						if (chord[2] = 0) and (p < chord[1]) then
							begin
								chord[2] := p;
								tar := tar - [p]
							end;
						if (chord[3] = 0) and (p < chord[2]) then
							begin
								chord[3] := p;
								tar := tar - [p]
							end;
						if (chord[4] = 0) and (p < chord[3]) then
							begin
								chord[4] := p;
								tar := tar - [p]
							end
					end;
			if tar <> [] then
				begin
					write('too many notes! ');
					exit(giveprelposfor)
				end;
{---------don't take open strings into account---------}
			testpositive := TRUE;
			for strings := 1 to 4 do
				prelfin[strings] := 99;
			for p := 1 to 4 do
				if (chord[p] >= tuning[1]) and (prelfin[1] = 99) then
					prelfin[1] := chord[p] - tuning[1]
				else if (chord[p] >= tuning[2]) and (prelfin[2] = 99) then
					prelfin[2] := chord[p] - tuning[2]
				else if (chord[p] >= tuning[3]) and (prelfin[3] = 99) then
					prelfin[3] := chord[p] - tuning[3]
				else if (chord[p] >= tuning[4]) and (prelfin[4] = 99) then
					prelfin[4] := chord[p] - tuning[4]
				else if chord[p] > 0 then
					testpositive := FALSE;
			closegap(prelfin);
			posit := 0;
			if testpositive then
				for pos := 17 downto 1 do
					for strings := 1 to 4 do
						if (prelfin[strings] > 0) and (pos = prelfin[strings]) then
							posit := pos;
			ps := [posit];
			exceptstrings := [];
			for strings := 1 to 4 do
				if prelfin[strings] = 99 then
					exceptstrings := exceptstrings + [strings];
{--------where possible, assign a tone to an open string-------}
			for p := 1 to 4 do
				for strings := 1 to 4 do
					if chord[p] = tuning[strings] then
						begin
							testpositive := TRUE;
							for str := 1 to 4 do
								prelfin[str] := 99;
							cord := chord;
							prelfin[strings] := 0;
							cord[p] := 0;
							for q := 1 to 4 do
								if cord[q] > 0 then
									if (cord[q] >= tuning[1]) and (prelfin[1] = 99) then
										prelfin[1] := cord[q] - tuning[1]
									else if (cord[q] >= tuning[2]) and (prelfin[2] = 99) then
										prelfin[2] := cord[q] - tuning[2]
									else if (cord[q] >= tuning[3]) and (prelfin[3] = 99) then
										prelfin[3] := cord[q] - tuning[3]
									else if (cord[q] >= tuning[4]) and (prelfin[4] = 99) then
										prelfin[4] := cord[q] - tuning[4]
									else if cord[q] > 0 then
										testpositive := FALSE;
							closegap(prelfin);
							posit := 0;
							if testpositive then
								for pos := 17 downto 1 do
									for str := 1 to 4 do
										if (prelfin[str] > 0) and (pos = prelfin[str]) then
											posit := pos;
							ps := ps + [posit];
							compexceptstrings := [];
							for str := 1 to 4 do
								if prelfin[str] = 99 then
									compexceptstrings := compexceptstrings + [str];
							exceptstrings := exceptstrings * compexceptstrings
						end;
{--------where possible, assign two tones to two open strings-------}
			for p := 1 to 3 do
				for pp := p + 1 to 4 do
					for strings := 1 to 3 do
						for sstrings := strings + 1 to 4 do
							if (chord[p] = tuning[strings]) and (chord[pp] = tuning[sstrings]) then
								begin
									testpositive := TRUE;
									for str := 1 to 4 do
										prelfin[str] := 99;
									cord := chord;
									prelfin[strings] := 0;
									prelfin[sstrings] := 0;
									cord[p] := 0;
									cord[pp] := 0;
									for q := 1 to 4 do
										if cord[q] > 0 then
											if (cord[q] >= tuning[1]) and (prelfin[1] = 99) then
												prelfin[1] := cord[q] - tuning[1]
											else if (cord[q] >= tuning[2]) and (prelfin[2] = 99) then
												prelfin[2] := cord[q] - tuning[2]
											else if (cord[q] >= tuning[3]) and (prelfin[3] = 99) then
												prelfin[3] := cord[q] - tuning[3]
											else if (cord[q] >= tuning[4]) and (prelfin[4] = 99) then
												prelfin[4] := cord[q] - tuning[4]
											else if cord[q] > 0 then
												testpositive := FALSE;
									closegap(prelfin);
									posit := 0;
									if testpositive then
										for pos := 17 downto 1 do
											for str := 1 to 4 do
												if (prelfin[str] > 0) and (pos = prelfin[str]) then
													posit := pos;
									ps := ps + [posit];
									compexceptstrings := [];
									for str := 1 to 4 do
										if prelfin[str] = 99 then
											compexceptstrings := compexceptstrings + [str];
									exceptstrings := exceptstrings * compexceptstrings
								end;
{--------assign as many tones as possible to open strings------------}
			testpositive := TRUE;
			for strings := 1 to 4 do
				prelfin[strings] := 99;
			cord := chord;
			for p := 1 to 4 do
				for strings := 1 to 4 do
					if cord[p] = tuning[strings] then
						begin
							prelfin[strings] := 0;
							cord[p] := 0
						end;
			for p := 1 to 4 do
				if cord[p] > 0 then
					if (cord[p] >= tuning[1]) and (prelfin[1] = 99) then
						prelfin[1] := cord[p] - tuning[1]
					else if (cord[p] >= tuning[2]) and (prelfin[2] = 99) then
						prelfin[2] := cord[p] - tuning[2]
					else if (cord[p] >= tuning[3]) and (prelfin[3] = 99) then
						prelfin[3] := cord[p] - tuning[3]
					else if (cord[p] >= tuning[4]) and (prelfin[4] = 99) then
						prelfin[4] := cord[p] - tuning[4]
					else if cord[p] > 0 then
						testpositive := FALSE;
			closegap(prelfin);
			posit := 0;
			if testpositive then
				for pos := 17 downto 1 do
					for strings := 1 to 4 do
						if (prelfin[strings] > 0) and (pos = prelfin[strings]) then
							posit := pos;
			ps := ps + [posit];
			compexceptstrings := [];
			for str := 1 to 4 do
				if prelfin[str] = 99 then
					compexceptstrings := compexceptstrings + [str];
			exceptstrings := exceptstrings * compexceptstrings;
{-----check for perfect fifths-----------}
			dofifths := FALSE;
			q := 0;
			for p := bottomnote to topnote do
				if p in targetch then
					begin
						if p - q = 7 then
							dofifths := TRUE;
						q := p
					end;
{------check for open strings possibilities--------}
			dozeroson := [];
			for str := 1 to 4 do
				if tuning[str] in targetch then
					dozeroson := dozeroson + [str];
		end;{giveprelposfor}

		procedure report (arr: lefthand);
			var
				str, c: byte;
				cs, eset: pitchset;
				arrempty, arrthere: boolean;
		begin{report}
			cs := [];
			for str := 1 to 4 do
				if not (arr[str] = 99) then
					cs := cs + [tuning[str] + arr[str]];
			if targetch <= cs then
				begin
					eset := [];
					for str := 1 to 4 do
						if (arr[str] = 99) or (arr[str] = 0) then
							eset := eset + [str];
					eset := eset + fiftheds;
					if strins = vlc then
						begin
{---------Q-position vs normal position--------------------}
							if (incr2 > 2) or ((incr3 <> 1) or (incr4 <> 1)) then
								begin
									if alerts mod 13 <> 0 then
										alerts := alerts * 13
								end
							else if alerts mod 2 <> 0 then
								alerts := alerts * 2;
{-----------------hindered open string--------------------}
							for str := 1 to 2 do
								if (arr[str] = 0) and (not (str + 1 in eset) and (arr[str + 1] = arr[str + 2])) then
									if (alerts mod 2 = 0) and (alerts mod 4 <> 0) then
										alerts := alerts * 2
						end;
					if (strins in [vl1, vl2]) or (strins = vla) then
						begin
{-----two fingers in same halfposition-------------}
							if arr[1] = arr[3] then
								if not (1 in eset) then
									if alerts mod 3 <> 0 then
										alerts := alerts * 3;
							if arr[1] = arr[4] then
								if not (1 in eset) then
									if alerts mod 3 <> 0 then
										alerts := alerts * 3;
							if arr[2] = arr[4] then
								if not (2 in eset) then
									if alerts mod 3 <> 0 then
										alerts := alerts * 3;
{----------------twice stretched--------------------}
							if (incr2 = 3) and (incr3 = 3) then
								if not (((f1 in eset) or (f2 in eset)) or (f3 in eset)) then
									if alerts mod 5 <> 0 then
										alerts := alerts * 5;
							if (incr2 = 3) and (incr4 = 3) then
								if not (((f1 in eset) or (f2 in eset)) or ((f3 in eset) or (f4 in eset))) then
									if alerts mod 5 <> 0 then
										alerts := alerts * 5;
							if (incr3 = 3) and (incr4 = 3) then
								if not ((f2 in eset) or ((f3 in eset) or (f4 in eset))) then
									if alerts mod 5 <> 0 then
										alerts := alerts * 5;
{------------very wide stretching-------------------}
							if incr2 = 4 then
								if not ((f1 in eset) or (f2 in eset)) then
									if alerts mod 7 <> 0 then
										alerts := alerts * 7;
							if incr3 = 4 then
								if not ((f2 in eset) or (f3 in eset)) then
									if alerts mod 7 <> 0 then
										alerts := alerts * 7;
							if incr4 = 4 then
								if not ((f3 in eset) or (f4 in eset)) then
									if alerts mod 7 <> 0 then
										alerts := alerts * 7
						end;
					c := 0;
					repeat
						arrempty := TRUE;
						arrthere := TRUE;
						c := c + 1;
						for str := 1 to 4 do
							begin
								if inventory[c][str] <> arr[str] then
									arrthere := FALSE;
								if inventory[c][5] <> alerts then
									arrthere := FALSE;
								if inventory[c][str] <> 99 then
									arrempty := FALSE
							end
					until (c = 8) or (arrempty or arrthere);
					if (c = 8) and not arrempty then
						taskdone := TRUE;
					if (not arrthere) and arrempty then
						begin
							for str := 1 to 4 do
								inventory[c][str] := arr[str];
							inventory[c][5] := alerts
						end
				end
		end;{report}

		procedure openstrings (arr: lefthand;
										doneon: pitchset);
			var
				str, sstr: byte;
				fingarr: lefthand;
		begin{openstrings}
			fingarr := arr;
			for str := 1 to 4 do
				if str in doneon then
					fingarr[str] := 0;
			report(fingarr);                          { all zeroed  }
			for str := 1 to 4 do
				if str in doneon then
					begin
						fingarr[str] := arr[str];
						report(fingarr);                        { just one not zeroed  }
						fingarr[str] := 0
					end;
			for str := 1 to 3 do
				if str in doneon then
					for sstr := str + 1 to 4 do
						if sstr in doneon then
							begin
								fingarr := arr;
								fingarr[str] := 0;
								fingarr[sstr] := 0;
								report(fingarr)                        { just two zeroed  }
							end;
			for str := 1 to 4 do
				if str in doneon then
					begin
						fingarr := arr;
						fingarr[str] := 0;
						report(fingarr)                        { just one zeroed }
					end;
		end;{openstrings}

		procedure lessfingers (arr: lefthand);
			var
				fingarray: lefthand;
				done: boolean;
				doiton: pitchset;
				str: byte;
		begin{lessfingers}
{------make fifth, using index finger-----}
			if (strins in [vl1, vl2]) or (strins = vla) then
				if halfposition < 8 then
					begin
						fingarray := arr;
						doiton := dozeroson;
						if f1 <> 4 then
							if ((f1 <> 1) or (fingarray[3] <> fingarray[1])) and ((f1 <> 2) or (fingarray[4] <> fingarray[2])) then
								if fingarray[f1 + 1] <> 99 then
									begin
										fingarray[f1 + 1] := fingarray[f1];
										fiftheds := [f1, f1 + 1];
										doiton := doiton - fiftheds;
										report(fingarray);
										openstrings(fingarray, doiton)
									end;
{------make fifth, using middle finger---------}
						fingarray := arr;
						doiton := dozeroson;
						if f2 <> 4 then
							if ((f2 <> 1) or (fingarray[3] <> fingarray[1])) and ((f2 <> 2) or (fingarray[4] <> fingarray[2])) then
								if fingarray[f2 + 1] <> 99 then
									begin
										fingarray[f2 + 1] := fingarray[f2];
										fiftheds := [f2, f2 + 1];
										doiton := doiton - fiftheds;
										report(fingarray);
										openstrings(fingarray, doiton)
									end;
{-----next, the index fifth plus the middle finger fifth----}
						fingarray := arr;
						if (((f1 = 1) and (f2 = 3)) or ((f1 = 3) and (f2 = 1))) then
							if ((fingarray[f1 + 1] <> 99) and (fingarray[f2 + 1] <> 99)) then
								if fingarray[1] <> fingarray[3] then
									begin
										fingarray[f1 + 1] := fingarray[f1];
										fingarray[f2 + 1] := fingarray[f2];
										fiftheds := [1, 2, 3, 4];
										report(fingarray)
									end
					end;
{-------------------vlc: fifths for all fingers-------------------}
			if strins = vlc then
				begin
					for str := 1 to 3 do
						begin
							fingarray := arr;
							doiton := dozeroson;
							if (fingarray[str] <> 99) and (fingarray[str + 1] <> 99) then
								begin
									fingarray[str + 1] := fingarray[str];
									doiton := doiton - [str, str + 1];
									report(fingarray);
									openstrings(fingarray, doiton)
								end
						end;
{-------------------vlc: two fifths-------------------------------}
					fingarray := arr;
					if ((fingarray[1] <> 99) and (fingarray[2] <> 99)) and ((fingarray[3] <> 99) and (fingarray[4] <> 99)) then
						begin
							fingarray[2] := fingarray[1];
							fingarray[4] := fingarray[3];
							report(fingarray)
						end
				end
		end;{lessfingers}

		procedure barrechords (arr: lefthand);
			var
				str, sstr, ssstr: byte;
				fingarr: lefthand;
				doiton: pitchset;
		begin{barrechords}
			fingarr := arr;
			if f1 = 4 then
				doiton := []
			else
				doiton := dozeroson - [3, 2, 1];
			for str := 1 to f1 - 1 do
				if not (fingarr[str] = 99) then
					for sstr := 1 to f1 - 1 do
						if not (fingarr[sstr] = 99) then
							for ssstr := 1 to f1 - 1 do
								if not (fingarr[ssstr] = 99) then
									begin
										fingarr[str] := fingarr[f1];
										fingarr[sstr] := fingarr[f1];
										fingarr[ssstr] := fingarr[f1];
										if alerts mod 11 <> 0 then
											alerts := alerts * 11;
										report(fingarr);
										openstrings(fingarr, doiton);
										fingarr := arr
									end;
			if f2 = 3 then
				doiton := dozeroson - [3, 2, 1];
			for str := 1 to f2 - 1 do
				if not (fingarr[str] = 99) then
					for sstr := 1 to f2 - 1 do
						if not (fingarr[sstr] = 99) then
							begin
								fingarr[str] := fingarr[f2];
								fingarr[sstr] := fingarr[f2];
								if alerts mod 11 <> 0 then
									alerts := alerts * 11;
								report(fingarr);
								openstrings(fingarr, doiton);
								fingarr := arr
							end
		end;{barrechords}

		procedure giveresults (r: resultlist);
			var
				c, str, halfp, handw, difficulty: byte;
				arrempty: boolean;
		begin
{writeln('RESULTLIST:');    for c := 1 to 8 do    begin}
{for str := 5 downto 1 do    write(r[c][str] : 2, ' ');    writeln    end;}
			c := 1;
			difficulty1 := 255;
			difficulty2 := 255;
			handposition1 := 255;
			handposition2 := 255;
			repeat
				arrempty := TRUE;
				for str := 1 to 5 do
					if r[c][str] <> 99 then
						arrempty := FALSE;
				difficulty := 255;
				halfp := 15;
				if not arrempty then
					begin
{----look for the lowest halfpos and the width the hand has to span----}
						halfp := 15;{find the lowest halfposition in this fingering}
						for str := 1 to 4 do
							if r[c][str] < halfp then
								if r[c][str] > 0 then
									halfp := r[c][str];
{---- if there is nothing between 0 and 99 (the nolefthand case) then it is an all open strings case -------}
						if halfp = 15 then
							halfp := 0;
						handw := 0;{find the required minimum width for the hand to span}
						for str := 1 to 4 do
							if r[c][str] > halfp then
								if r[c][str] < 99 then
									if (r[c][str] - halfp) > handw then
										handw := r[c][str] - halfp;
{----take in the complications----}
						difficulty := 0;
						if r[c][5] = 0 then {the all open strings case}
							difficulty := difficulty + 1
						else
							begin
								if r[c][5] mod 2 = 0 then {normal position: 1 in pos 1 from end of binary form}
									difficulty := difficulty + 2;
								if r[c][5] mod 3 = 0 then {two fingers in the same halfposition: 1 in pos 5 from end in binary form}
									difficulty := difficulty + 32;
								if r[c][5] mod 4 = 0 then {open string may be hindered: 1 in pos 4 from end in binary form}
									difficulty := difficulty + 16;
								if r[c][5] mod 5 = 0 then {twice stretched: 1 in pos 7 from end in binary form}
									difficulty := difficulty + 128;
								if r[c][5] mod 7 = 0 then {very wide stretching: 1 in pos 6 from end in binary form}
									difficulty := difficulty + 64;
								if r[c][5] mod 11 = 0 then {barr chord: 1 in pos 3 from end in binary form}
									difficulty := difficulty + 8;
								if r[c][5] mod 13 = 0 then {Q position: 1 in pos 2 from end in binary form}
									difficulty := difficulty + 4;
							end
					end;
				if (difficulty < difficulty1) or ((difficulty = difficulty1) and (halfp < posvalue(2, difficulty1, 16))) then
					begin {---------- find best of all ---------------}
{writeln('fingering ', c : 1, ' gets into the first slot: ', difficulty : 2, ' ', halfp : 2, ' ', handw : 2);}
						difficulty2 := difficulty1;
						handposition2 := handposition1;
						difficulty1 := difficulty;
						handposition1 := hextobyte(halfp, handw)
					end
				else if (difficulty < difficulty2) or ((difficulty = difficulty2) and (halfp < posvalue(2, difficulty2, 16))) then
					begin {-----------find second best------------}
{writeln('fingering ', c : 1, ' gets into the second slot: ', difficulty : 2, ' ', halfp : 2, ' ', handw : 2);}
						difficulty2 := difficulty;
						handposition2 := hextobyte(halfp, handw)
					end;
				c := c + 1
			until arrempty or (c = 9);{ ------never more than eight solutions listed(!)-------}
		end;

		function feasible (fi: lefthand): boolean;
		begin{feasible}
			feasible := TRUE;
			if (strins in [vl1, vl2]) or (strins = vla) then
				begin
					if fi[1] = fi[2] then
						if not (fi[1] = 99) then
							feasible := FALSE;
					if fi[2] = fi[3] then
						if not (fi[2] = 99) then
							feasible := FALSE;
					if fi[3] = fi[4] then
						if not (fi[3] = 99) then
							feasible := FALSE;
					if (incr2 + incr3 + incr4 > 7) and (halfposition < 5) then
						feasible := FALSE;
					if incr2 + incr3 + incr4 > 8 then
						feasible := FALSE
				end;
			if strins = vlc then
				case halfposition of
					1..3: 
						if incr2 + incr3 + incr4 > 5 then
							feasible := FALSE;
					4..6: 
						if incr2 + incr3 + incr4 > 6 then
							feasible := FALSE;
					7, 8: 
						if incr2 + incr3 + incr4 > 7 then
							feasible := FALSE;
					otherwise
						if incr2 + incr3 + incr4 > 8 then
							feasible := FALSE;
				end;
		end;{feasible}

	begin{difficulty}
		if card(targetch) > 1 then
			begin
				taskdone := FALSE;
{----------initialize array to zero-------------}
				for a := 1 to 8 do
					for str := 1 to 5 do
						inventory[a][str] := 99;
{----------specify the tuning------------------}
				case strins of
					vl1, vl2: 
						begin
							tuning[1] := middlec + 16;
							tuning[2] := middlec + 9;
							tuning[3] := middlec + 2;
							tuning[4] := middlec - 5;
							i := 0;
							ii := i;
							ij := 4;
							j := 4
						end;
					vla: 
						begin
							tuning[1] := middlec + 9;
							tuning[2] := middlec + 2;
							tuning[3] := middlec - 5;
							tuning[4] := middlec - 12;
							i := 0;
							ii := i;
							ij := 4;
							j := 3
						end;
					vlc: 
						begin
							tuning[1] := middlec - 3;
							tuning[2] := middlec - 10;
							tuning[3] := middlec - 17;
							tuning[4] := middlec - 24;
							i := 1
						end
				end;
{-----see whether the chord can be done without any left hand at all...------}
				tuningset := [];
				for a := 1 to 4 do
					tuningset := tuningset + [tuning[a]];
				if targetch <= tuningset then
					begin
						nolefthand := TRUE;
						a := tuning[4] - 1;
						b := 0;
						repeat
							repeat
								a := a + 1
							until (a in targetch) or (a > tuning[1]);
							if a in targetch then
								begin
									if (b > 0) and (a - b <> 7) then
										nolefthand := FALSE;
									b := a
								end
						until a > tuning[1];
						if nolefthand then
							begin {zzzzzz}
								for str := 1 to 4 do
									if tuning[str] in targetch then
										inventory[1][str] := 0;
								inventory[1][5] := 0
							end
					end;
{-------------work your way through all relevant positions---------------}
				giveprelposfor(targetch, pset);
				for halfposition := 1 to 17 do
					if halfposition in pset then
						begin
							if strins = vlc then
								case halfposition of
									1..3: 
										begin
											ii := 0;
											ij := 3;
											j := 2
										end;
									4..6: 
										begin
											ii := 0;
											ij := 4;
											j := 2
										end;
									7, 8: 
										begin
											ii := 0;
											ij := 4;
											j := 2
										end;
									otherwise
										begin
											ii := 0;
											ij := 5;
											j := 2
										end;
								end;
{----------through all distance relations between fingers------------}
							for incr2 := i to ij do
								for incr3 := ii to j do
									for incr4 := ii to j do
{----------through all distributions of fingers over strings---------}
										for f1 := 1 to 4 do
											for f2 := 1 to 4 do
												if f2 <> f1 then
													for f3 := 1 to 4 do
														if (f3 <> f2) and (f3 <> f1) then
															for f4 := 1 to 4 do
																if ((f4 <> f3) and (f4 <> f2)) and (f4 <> f1) then
																	begin
																		fingering[f1] := halfposition;
																		fingering[f2] := fingering[f1] + incr2;
																		fingering[f3] := fingering[f2] + incr3;
																		fingering[f4] := fingering[f3] + incr4;
																		if not taskdone then
																			if feasible(fingering) then
																				begin
																					for str := 1 to 4 do
																						if str in exceptstrings then
																							fingering[str] := 99;
																					alerts := 1;
																					fiftheds := [];
																					report(fingering);
																					dozeroson := dozeroson - exceptstrings;
																					openstrings(fingering, dozeroson);
																					if dofifths then
																						lessfingers(fingering);
																					if strins = vlc then
																						if ((halfposition < 12) and (incr2 < 3)) and ((incr3 = 1) and (incr4 = 1)) then
																							if (f1 >= 3) or (f2 >= 3) then
																								barrechords(fingering)
																				end;
																	end
						end;
				giveresults(inventory)
			end
		else
			begin
				givelimitsof(strins, lowest, low, high, highest);
				p := bottomnote - 1;
				repeat
					p := p + 1
				until (p in targetch) or (p > topnote);
				if p in targetch then
					if (p <= highest) and (p >= lowest) then
						begin { tone is within range }
							difficulty1 := 0;
							handposition1 := 255;
							difficulty2 := 0;
							handposition2 := 255
						end
					else
						begin { tone is out of range of this instrument }
							difficulty1 := 255;
							handposition1 := 255;
							difficulty2 := 255;
							handposition2 := 255
						end
				else  { "tone" is out of all and any musical range }
					writeln('What "tone" do you think you'' re working on ?')
			end
	end;{difficulty}

end.