{	This program is copyright (c) 1993 - 2005 Jos Kunst, the Jos Kunst heirs.	}
{	This program is free software. You can redistribute it and/or modify it 	}
{	under the terms of version 2 of the GNU General Public License, which you	}
{	should have received with it.												}
{	This program is distributed in the hope that it will be useful, but			}
{	without any warranty, expressed or implied.									}

unit productions;

interface
	uses
		translations, chordscanner, chordworker, randomizers, outputs, players, workspace, doposition;

	const
		fibfactor = 1.618034; { sizeable step  }

	var
		enough: boolean;

	function staminalength (lastlength: real;
									stamina: byte): real;
{regulates acceleration and deceleration rates}
	function nearestoctaveof (anchor: byte;
									cs: pitchset): byte;
{given a tone T and a chord C: finds the note belonging to T's pitch class that is closest to any note of C}
	function unisono (ins: instruments;
									pitch: byte;
									pos: position): boolean;
{for a given instrument I, tone T, and position P, finds out whether some instrument other than I has T in P}
	procedure givewidthandcenter (cs: pitchset;
									var width: byte;
									var center: byte);
{for a given chord C, gives C's width and center}
	procedure reshuffle (var cs: pitchset);
{randomly reshuffles the chord's tones and then restores the original dissonance}
	procedure relocate (var cs: pitchset;
									chordstep: integer;
									rroom: byte;
									var result: integer);
{given a step S and a random space R, moves a given chord C, for as far as instrumentally possible, }
{over a distance S, and with a random variation over R}
	procedure melodyevent (instr: instruments;
									fromnote, tonote: byte;
									var lincho: pitchset;
									var lapo: position);
{changes a chord and a position by moving a part from a note to another note and fixing it there}
	procedure limitedreshuffle (var cs: pitchset);
{changes a chord by making each of its tones go through a (major or minor) second step}
	procedure chordprogression (var lapo: position;
									var memspan: chintarr;
									var lincho: pitchset;
									spaceshift: integer;
									randroom: byte;
									dsgstep: real;
									howlong: byte);
{changes a chord by }
{  1) modifiying its position, putting it through a global moving procedure (with room for a random factor),  }
{  2) working on its dsg, by either }
{  2a) using one calculated step-length in dsg-space, if "dsgstep" is between 9 and -9, or (else)  }
{  2b) using a given number of psm steps, if abs("dsgstep") is between 10 and 19; or, else still,  }
{  2c) putting it through a number of 'minimal' dsg steps, if abs("dsgstep") is between 20 and 99; or, else still,   }
{  2d) changing the dsg by a given amount, if abs("dsgstep") is between 100 and 299, where  }
{  2e) one first puts it through a limited reshuffle, if abs("dsgstep") is between 200 and 299. }
	procedure checkrecurring (var mmsp: chintarr;
									cs: pitchset);
{checks whether any chord, defined as an interval constellation, }
{recurs for the third time in a sixteen items chord chain}

implementation

	function staminalength (lastlength: real;
									stamina: byte): real;
{---------- stamina = (maxfactor - 1) / (minfactor - 1) ----------------}
		var
			stamen: integer;
			minfactor, length, lengthfactor: real;
			accel, tighten: boolean;

		function fibonacci (num: byte): integer;
{------------num groter dan 21 overschrijdt de integer bovengrens! ---------}
			var
				count: byte;
				a, b, c: integer;
		begin{fibonacci}
			a := 0;
			b := 1;
			c := a + b;
			for count := 1 to num do
				begin
					a := b;
					b := c;
					c := a + b
				end;
			fibonacci := c
		end;{fibonacci}

	begin{staminalength}
		stamen := fibonacci(stamina);
		minfactor := (stamen + (maxfactor - 1)) / stamen;
		lengthfactor := maxfactor - (maxfactor - minfactor) * (1 - (length - minlength) / (maxlength - minlength));
		if length <= minlength then
			accel := FALSE;
		if length >= maxlength then
			accel := TRUE;
		if accel then
			lengthfactor := 1 / lengthfactor;
		staminalength := lastlength * lengthfactor
	end;{staminalength}

	function nearestoctaveof (anchor: byte;
									cs: pitchset): byte;
		var
			p, q, c, dist, disst, closest: byte;
	begin
		p := anchor;
		while p <= topnote do
			p := p + 12;
		for c := bottomnote to topnote do
			if c in cs then
				q := c;
		while abs(p - q) > 24 do
			p := p - 12;
		dist := 13;
		repeat
			p := p - 12;
			for c := bottomnote to topnote do
				if c in cs then
					if abs(c - p) < dist then
						begin
							dist := abs(c - p);
							closest := c
						end;
		until p < bottomnote + 12;
		repeat
			disst := abs(p - closest);
			if disst = dist then
				nearestoctaveof := p;
			p := p + 12
		until disst = dist
	end;

	function unisono (ins: instruments;
									pitch: byte;
									pos: position): boolean;
		var
			i: instruments;
			c: byte;
	begin
		unisono := FALSE;
		for i := vl1 to vlc do
			for c := 1 to 5 do
				if (pos[ord(i) * 5 + c, 1] = pitch) and (i <> ins) then
					unisono := TRUE;
	end;

	procedure givewidthandcenter (cs: pitchset;
									var width: byte;
									var center: byte);
		var
			p, bottom, top: byte;
			bottomfound: boolean;
	begin
		bottom := 0;
		top := 0;
		bottomfound := FALSE;
		for p := bottomnote to topnote do
			if (p in cs) then
				if not bottomfound then
					begin
						bottom := p;
						bottomfound := TRUE;
						top := bottom
					end
				else
					top := p;
		width := top - bottom;
		center := bottom + width div 2
	end;

	procedure reshuffle (var cs: pitchset);
		var
			p, num: byte;
			newcs: pitchset;
	begin
		newcs := [];
		for p := bottomnote to topnote do
			if p in cs then
				begin
					num := card(newcs);
					repeat
						newcs := newcs + [p - 6 + randnum(12)]
					until card(newcs) > num
				end;
		cs := newcs
	end;

	procedure limitedreshuffle (var cs: pitchset);
		var
			p, r, oldcard: byte;
			newcs: pitchset;
	begin
		newcs := [];
		for p := bottomnote to topnote do
			if p in cs then
				begin
					oldcard := card(newcs);
					repeat
						r := randnum(4);
						case r of
							1: 
								newcs := newcs + [p + 1];
							2: 
								newcs := newcs + [p - 1];
							3: 
								newcs := newcs + [p + 2];
							4: 
								newcs := newcs + [p - 2]
						end
					until card(newcs) > oldcard
				end;
		cs := newcs
	end;

	procedure relocate (var cs: pitchset;
									chordstep: integer;
									rroom: byte;
									var result: integer);
		var
			p, w, c, cc, r, ensemblewidth, dice: byte;
			spacerat: real;
	begin
		givewidthandcenter(cs, w, c);
		ensemblewidth := ensembletop - ensemblebottom;
		cc := c;
		c := c + chordstep;
		if c + round(w / 2) > ensembletop then
			c := ensembletop - round(w / 2);
		if c - round(w / 2) < ensemblebottom then
			c := ensemblebottom + round(w / 2);
		spacerat := (c - (ensemblebottom + w / 2)) / (ensemblewidth - w);
{-------say: downward elbow room divided by total elbow room... -------}
{------... or: how high is it with respect to where it could have been?-----------}
		r := rroom;
		if r >= ensemblewidth - w then
			if w <= ensemblewidth then
				r := ensemblewidth - w
			else
				r := 0;
		c := c - round(spacerat * r);
{-------... i.e. if the chord is as high as it can go, all of its elbow room should be below it, ----------}
{-------... if it is as low as possible, all of its elbow room should be above it, ----------}
{-------... and all intermediate cases should be handled proportionally. ------------}
		if r >= 1 then
			dice := randnum(r + 1) - 1
		else
			dice := 0;
		c := c + dice;
		if c + round(w / 2) > ensembletop then
			c := ensembletop - round(w / 2);
		if c - round(w / 2) < ensemblebottom then
			c := ensemblebottom + round(w / 2);
		if c > cc then
			for p := topnote downto bottomnote do
				if p in cs then
					begin
						cs := cs - [p];
						cs := cs + [p + (c - cc)]
					end;
		if cc > c then
			for p := bottomnote to topnote do
				if p in cs then
					begin
						cs := cs - [p];
						cs := cs + [p - (cc - c)]
					end;
		result := c - cc
	end;

{----------------------------- ... paardemiddel... -------------------------}
{-------- if tonote is 1 then doposition will see to it that one hand will be freed ---------}
{-------------if tonote = 0 then there will be no melfix and both hands will be available -------}
{----------- if tonote is between bottomnote and topnote there will be a melfix effect -----------}
	procedure melodyevent (instr: instruments;
									fromnote, tonote: byte;
									var lincho: pitchset;
									var lapo: position);
	begin
{if not unisono(instr, fromnote, lapo) then}
		lincho := lincho - [fromnote];
		if tonote < bottomnote then
			if tonote > 1 then
				tonote := 0;
		if tonote > topnote then
			tonote := 0;
		if tonote > bottomnote then
			lincho := lincho + [tonote];
		melfix := tonote;
		lapo[ord(instr) + 1, 1] := tonote;
		if tonote > 0 then
			lapo[ord(instr) + 1, 3] := 255
		else
			lapo[ord(instr) + 1, 3] := 0
	end;

	procedure chordprogression (var lapo: position;
									var memspan: chintarr;
									var lincho: pitchset;
									spaceshift: integer;
									randroom: byte;
									dsgstep: real;
									howlong: byte);
		var
			c, p, count, cont, pasthroot, currentdsg, disstarget: byte;
			abstep, restep, dissstep, fraction, factor, substitdsg: real;
			thpo: position;
			resultstep: integer;
			cr: chordrec;

		procedure checkrecurring (var mmsp: chintarr;
										cs: pitchset);
			var
				p, q, r, s: byte;
				sofartta, done: boolean;
				chinta: array[1..12] of byte;
		begin
{---- translate pitchset to array of intervals ----}
			for p := 1 to 12 do
				chinta[p] := 0;
			for p := bottomnote to topnote do
				if p in cs then
					begin
						q := p;
						repeat
							q := q + 1;
							if q in cs then
								begin
									done := FALSE;
									r := 0;
									repeat
										r := r + 1;
										if chinta[r] = 0 then
											begin
												chinta[r] := q - p;
												done := TRUE
											end
									until done
								end
						until (q in cs) or (q > topnote)
					end;
{------ store interval array in rightmost position in memoryspace --------}
			for p := 1 to 15 do
				for q := 1 to 12 do
					mmsp[p, q] := mmsp[p + 1, q];
			for r := 1 to 12 do
				mmsp[16, r] := chinta[r];
{-------write the relevant part of the memory space to the screen--------}
			if showmem then
				for q := card(cs) - 1 downto 1 do
					begin
						for p := 1 to 16 do
							write(mmsp[p, q] : 3);
						writeln
					end;
{----- check whether any chord has occurred three times --------}
			sofartta := FALSE;
			p := 17;
			repeat
				p := p - 1;
				q := p;
				repeat
					q := q - 1;
					if (mmsp[p, 1] > 0) and (mmsp[p, 1] = mmsp[q, 1]) then
						begin
							r := q;
							repeat
								r := r - 1;
								if mmsp[q, 1] = mmsp[r, 1] then
									begin
										sofartta := TRUE;
										s := 1;
										repeat
											s := s + 1;
											if (mmsp[p, s] <> mmsp[q, s]) or (mmsp[p, s] <> mmsp[r, s]) then
												sofartta := FALSE
										until (s = 12) or (sofartta = FALSE)
									end
							until (r = 1) or sofartta
						end
				until (q = 2) or sofartta
			until (p = 3) or sofartta;
			thirdtimearound := sofartta;
			if thirdtimearound then
				writeln('!!!')
		end;

	begin
		pasthroot := hroot;
		if (spaceshift <> 0) or (randroom > 0) then
			begin
				relocate(lincho, spaceshift, randroom, resultstep);
				updateanalysis(lincho, resultstep)
			end;
{============= the dsgsteps: the "measured step" cases: ================}
		if abs(dsgstep) <= 9 then
{------in case the target is just the current dsg-----------}
			if dsgstep = 0 then
				lincho := lincho
			else
{-------------using the step length, calculate the target dissonance--------}
				begin
					abstep := abs(dsgstep);
					dissstep := 1;
					c := 1;
					while c <= trunc(abstep) do
						begin
							dissstep := dissstep * fibfactor;
							c := c + 1
						end;
{----... takes care of the whole number part of the dsg step: so many times the whole fibfactor-----}
					restep := abstep - c + 1;
{------(c was one too high as a result!) ... remains the fraction part of the dsg step... --------}
					fraction := 1;
					factor := fibfactor;
					for count := 1 to 5 do
						begin
							fraction := 0.5 * fraction;
							factor := sqrt(factor);
							if restep >= fraction then
								begin
									dissstep := dissstep * factor;
									restep := restep - fraction
								end
						end;
{------ avoid getting trapped in the case of a dsg zero...--------------}
					if lapo[m, 4] = 0 then
						substitdsg := 0.5
					else
						substitdsg := lapo[m, 4];
					if dsgstep < 0 then
						targetsetto(lincho, round(substitdsg / dissstep))
					else
						targetsetto(lincho, round(substitdsg * dissstep))
				end
{============== the psm cases ===============}
		else if abs(dsgstep) <= 19 then
			if dsgstep < 0 then
				psmconsset(lincho, abs(trunc(dsgstep)) - 10)
			else
				psmdissset(lincho, abs(trunc(dsgstep)) - 10)
{=========== the minimum step cases ==============}
		else if abs(dsgstep) <= 99 then
			if dsgstep < 0 then
				minimumconsset(lincho, abs(trunc(dsgstep)) - 20)
			else
				minimumdissset(lincho, abs(trunc(dsgstep)) - 20)
{=========== the target dsg found by adding or subtracting operations =========}
		else if abs(dsgstep) <= 199 then
			if dsgstep < 0 then
				begin
 {--- avoid targets below zero ---}
					if dsg - (abs(trunc(dsgstep)) - 100) < 0 then
						disstarget := 0
					else
						disstarget := dsg - (abs(trunc(dsgstep)) - 100);
					targetsetto(lincho, disstarget)
				end
			else
				targetsetto(lincho, dsg + (abs(trunc(dsgstep)) - 100))
{=========== preceded by a limited reshuffling of the chord ==============}
		else if abs(dsgstep) <= 299 then
			begin
				currentdsg := dsg;
				limitedreshuffle(lincho);
				if dsgstep < 0 then
					begin
 {--- avoid targets below zero ---}
						if currentdsg - (abs(trunc(dsgstep)) - 200) < 0 then
							disstarget := 0
						else
							disstarget := currentdsg - (abs(trunc(dsgstep)) - 200);
						targetsetto(lincho, disstarget)
					end
				else
					targetsetto(lincho, currentdsg + (abs(trunc(dsgstep)) - 200))
			end
		else
			begin
				writeln('Sorry - no interpretation for dsgstep ', trunc(dsgstep) : 1, ' exists.');
				halt
			end;

		checkrecurring(memspan, lincho);
		if thirdtimearound then
			begin
				thirdtimearound := FALSE;
				for count := 1 to 16 do
					for cont := 1 to 12 do
						memspan[count, cont] := 0;
				currentdsg := dsg;
				reshuffle(lincho);
				targetsetto(lincho, currentdsg);
{ --- and store it in the last position of the now all-zero memory span array: ----}
				checkrecurring(memspan, lincho)
			end;

{-------- if the chord has not been reanalyzed in its present position (perhaps simply exchanged for another): ------}
		if dsgstep = 0 then
			begin
				scan(lincho, cr);
				carr := cr.stru;
				dsg := cr.diss;
				hroot := cr.hifu;
				totalofall := cr.total
			end;

{----------------------the preferthirds effect, if any---------------------}
		if melfix <= 1 then {waarom 1??... --> vanwege de optie een hele pianohand vrij te maken... dit moet anders zzzzzz}
			if preferthirds then
				begin
					c := abs(pasthroot - hroot) mod 12;
					case c of
						5, 10: 
							if hroot > pasthroot then
								begin
									relocate(lincho, -1, 0, resultstep);
									updateanalysis(lincho, resultstep)
								end
							else
								begin
									relocate(lincho, +1, 0, resultstep);
									updateanalysis(lincho, resultstep)
								end;
						2, 7: 
							if hroot > pasthroot then
								begin
									relocate(lincho, +1, 0, resultstep);
									updateanalysis(lincho, resultstep)
								end
							else
								begin
									relocate(lincho, -1, 0, resultstep);
									updateanalysis(lincho, resultstep)
								end;
						otherwise
							lincho := lincho
					end;
				end;

{    for c := bottomnote to topnote do    if c in lincho then    writeln(c);}
{---------------------------the melfix effect, if any-------------------------------}
{    if (melfix > 1) and not (nearestoctaveof(melfix, lincho) in lincho) then}
{    begin    pullbackchord(lincho, nearestoctaveof(melfix, lincho), resultstep);    updateanalysis(lincho, resultstep) end;}
		if melfix > 1 then
			begin
				pullbackchord(lincho, melfix, resultstep);
				updateanalysis(lincho, resultstep)
			end;

{write('new chord: ');  for c := bottomnote to topnote do  if c in lincho then write(nameof(c), ' '); writeln; if printfile then begin}
{write(orchords, 'new chord: '); for c := bottomnote to topnote do if c in lincho then write(orchords, nameof(c), ' '); }
{writeln(orchords) end;}

		if howlong > 254 then
			begin
				enough := TRUE;
				howlong := 254
			end;

		donextposition(lapo, lincho, howlong, thpo);
		lapo := thpo;
		showpos(lapo)
	end;

end.